/* cached_result.cpp - implementation of class wxPg::Cached_result for
    storing locally entire result of SQL query.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/db/cached_result.h>
#include<wxPg/db/database.h>
#include<wxPg/utils/type.h>
#include<wxPg/utils/utils.h>
#include<cstdlib>

wxPg::Cached_result::Cached_result()
: current_(0)
{}


wxPg::Cached_result::Cached_result(const wxPg::Cached_result &src){
    this->make_copy_(src);
}


wxPg::Cached_result&
wxPg::Cached_result::operator=(const wxPg::Cached_result &src){
    this->clear();
    this->make_copy_(src);
    return *this;
}


wxPg::Cached_result::~Cached_result(){
    this->clear();
}


void
wxPg::Cached_result::store(const Database &db, PGresult *res){
    this->clear();
    if(!res)
        return;
    unsigned fields = static_cast<unsigned>(PQnfields(res));
    if(fields == 0)
        return;
    unsigned rows = static_cast<unsigned>(PQntuples(res));
    if(rows == 0)
        return;

    //store data
    this->data_.resize(rows);
    for(unsigned i = 0; i < rows; ++i){
        this->data_[i].resize(fields);
        for(unsigned j = 0; j < fields; ++j){
            data_[i][j].value = db.char_to_str(PQgetvalue(res, i, j));
            data_[i][j].null = PQgetisnull(res, i, j);
        }
    }

    //store field indices and types
    this->field_types_.resize(fields);
    for(unsigned i = 0; i < fields; ++i){
        this->field_types_[i] = oid_to_type_simple(PQftype(res, i));
        this->field_indices_[wxString::FromAscii(PQfname(res, i))] = i;
    }

    this->current_ = 0;
}


void
wxPg::Cached_result::clear(){
    this->data_.clear();
    this->field_indices_.clear();
    this->field_types_.clear();
    this->current_ = 0;
}


int
wxPg::Cached_result::cols(){
    return this->field_indices_.size();
}


const wxString
wxPg::Cached_result::col_name(int idx){
    std::map<wxString, unsigned>::iterator
            i = this->find_field_idx_(idx);
    if(i != this->field_indices_.end())
        return i->first;

    return wxEmptyString;
}


int
wxPg::Cached_result::col_idx(const wxString &field){
    std::map<wxString, unsigned>::iterator
            i = this->field_indices_.find(field);
    if(i != this->field_indices_.end())
        return i->second;

    return -1;
}


int
wxPg::Cached_result::type_simple(int idx){
    if(!this->check_idx_(idx))
        return TYPE_UNKNOWN;

    return this->field_types_[idx];
}


int
wxPg::Cached_result::type_simple(const wxString &field){
    std::map<wxString, unsigned>::iterator
            i = this->field_indices_.find(field);

    if(i != this->field_indices_.end())
        return this->field_types_[i->second];

    return TYPE_UNKNOWN;
}


bool
wxPg::Cached_result::null(int idx){
    if(!this->check_data_(idx))
        return true;
    return this->data_[this->current_][idx].null;
}


bool
wxPg::Cached_result::null(const wxString &field){
    int idx = this->check_data_(field);
    if(idx < 0)
        return true;
    return this->data_[this->current_][idx].null;
}


int
wxPg::Cached_result::get_int(int idx){
    if(!this->check_data_(idx))
        return 0;
    return std::atoi(this->data_[this->current_][idx].value.char_str());
}


int
wxPg::Cached_result::get_int(const wxString &field){
    int idx = this->check_data_(field);
    if(idx < 0)
        return 0;
    return std::atoi(this->data_[this->current_][idx].value.char_str());
}


double
wxPg::Cached_result::get_float(int idx){
    if(!this->check_data_(idx))
        return 0.0;
    return std::atof(this->data_[this->current_][idx].value.char_str());
}


double
wxPg::Cached_result::get_float(const wxString &field){
    int idx = this->check_data_(field);
    if(idx < 0)
        return 0.0;
    return std::atof(this->data_[this->current_][idx].value.char_str());
}


const wxString
wxPg::Cached_result::get_str(int idx){
    if(!this->check_data_(idx))
        return wxEmptyString;
    return this->data_[this->current_][idx].value;
}


const wxString
wxPg::Cached_result::get_str(const wxString &field){
    int idx = this->check_data_(field);
    if(idx < 0)
        return wxEmptyString;
    return this->data_[this->current_][idx].value;
}


const wxDateTime
wxPg::Cached_result::get_date(int idx){
    wxDateTime result;
    if(!this->check_data_(idx))
        return result;
    result.ParseDate(this->data_[this->current_][idx].value);
    return result;
}


const wxDateTime
wxPg::Cached_result::get_date(const wxString &field){
    wxDateTime result;
    int idx = this->check_data_(field);
    if(idx < 0)
        return result;
    result.ParseDate(this->data_[this->current_][idx].value);
    return result;
}


const wxDateTime
wxPg::Cached_result::get_time(int idx){
    wxDateTime result;
    if(!this->check_data_(idx))
        return result;
    result.ParseTime(this->data_[this->current_][idx].value);
    return result;
}


const wxDateTime
wxPg::Cached_result::get_time(const wxString &field){
    wxDateTime result;
    int idx = this->check_data_(field);
    if(idx < 0)
        return result;
    result.ParseTime(this->data_[this->current_][idx].value);
    return result;
}


const wxDateTime
wxPg::Cached_result::get_date_time(int idx){
    wxDateTime result;
    if(!this->check_data_(idx))
        return result;
    result.ParseDateTime(this->data_[this->current_][idx].value);
    return result;
}


const wxDateTime
wxPg::Cached_result::get_date_time(const wxString &field){
    wxDateTime result;
    int idx = this->check_data_(field);
    if(idx < 0)
        return result;
    result.ParseDateTime(this->data_[this->current_][idx].value);
    return result;
}


bool
wxPg::Cached_result::get_bool(int idx){
    if(!this->check_data_(idx))
        return false;
    return wxPg::cstr_to_bool(
                    this->data_[this->current_][idx].value.char_str(),
                    this->field_types_[idx]);
}


bool
wxPg::Cached_result::get_bool(const wxString &field){
    int idx = this->check_data_(field);
    if(idx < 0)
        return false;
    return wxPg::cstr_to_bool(
                    this->data_[this->current_][idx].value.char_str(),
                    this->field_types_[idx]);
}


int
wxPg::Cached_result::rows(){
    return this->data_.size();
}


bool
wxPg::Cached_result::current(int idx){
    if(idx < 0)
        return false;
    if(idx >= this->rows())
        return false;
    this->current_ = idx;
    return true;
}


int
wxPg::Cached_result::current()const{
    return this->current_;
}


void
wxPg::Cached_result::make_copy_(const wxPg::Cached_result &src){
    this->field_indices_ = src.field_indices_;
    this->data_ = src.data_;
    this->field_types_ = src.field_types_;
    this->current_ = src.current_;
}


std::map<wxString, unsigned>::iterator
wxPg::Cached_result::find_field_idx_(int idx){
    if(!this->check_idx_(idx))
        return this->field_indices_.end();
    for(std::map<wxString, unsigned>::iterator
            i = this->field_indices_.begin();
            i != this->field_indices_.end(); ++i)
        if(i->second == static_cast<unsigned>(idx))
            return i;

    return this->field_indices_.end();
}


bool
wxPg::Cached_result::check_idx_(int idx){
    if(idx < 0)
        return false;
    if(static_cast<unsigned>(idx) >= this->field_indices_.size())
        return false;

    return true;
}


bool
wxPg::Cached_result::check_data_(int idx){
    if(this->rows() == 0)
        return false;
    if(!this->check_idx_(idx))
        return false;
    return true;
}


int
wxPg::Cached_result::check_data_(const wxString &field){
    if(this->rows() == 0)
        return -1;
    std::map<wxString, unsigned>::iterator
            i = this->field_indices_.find(field);
    if(i != this->field_indices_.end())
        return i->second;
    return -1;
}
