/* data_row.cpp - implementation of class wxPg::Data_row for accessing
    data from single row of SQL query result

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/db/data_row.h>
#include<wxPg/utils/type.h>
#include<wxPg/utils/utils.h>
#include<cstdlib>

wxPg::Data_row::Data_row(Database *db)
    : current_(0), result_(NULL), db_(db)
{}


wxPg::Data_row::~Data_row(){
    this->clear();
}


void
wxPg::Data_row::detach(){
    this->clear();
}


void
wxPg::Data_row::clear(){
    if(result_){
        PQclear(this->result_);
        this->result_ = NULL;
        this->db_->detach(this, wxPg::Database::ATTACH_TRANSACTION);
    }
}


int
wxPg::Data_row::cols(){
    if(!this->result_)
        return 0;

    return PQnfields(this->result_);
}


const wxString
wxPg::Data_row::col_name(int idx){
    if(!this->result_)
        return wxEmptyString;
    return wxString::FromAscii(PQfname(this->result_, idx));
}


int
wxPg::Data_row::col_idx(const wxString &field){
    if(!this->result_)
        return -1;
    return PQfnumber(this->result_, field.char_str());
}


int
wxPg::Data_row::type_simple(int idx){
    if(!this->check_col_idx_(idx))
        return wxPg::TYPE_UNKNOWN;

    return wxPg::oid_to_type_simple(PQftype(this->result_, idx));
}


int
wxPg::Data_row::type_simple(const wxString &field){
    int idx = this->col_idx(field);
    if(idx >= 0)
        return wxPg::oid_to_type_simple(PQftype(this->result_, idx));

    return wxPg::TYPE_UNKNOWN;
}


bool
wxPg::Data_row::null(int idx){
    if(!this->check_row_idx_())
        return true;
    if(!this->check_col_idx_(idx))
        return true;

    return PQgetisnull(this->result_, this->current_, idx);
}


bool
wxPg::Data_row::null(const wxString &field){
    if(!this->check_row_idx_())
        return true;
    int idx = this->col_idx(field);
    if(idx >= 0)
        return PQgetisnull(this->result_, this->current_, idx);

    return true;
}


int
wxPg::Data_row::get_int(int idx){
    if(!this->check_row_idx_())
        return 0;
    if(!this->check_col_idx_(idx))
        return 0;

    return std::atoi(PQgetvalue(this->result_, this->current_, idx));
}


int
wxPg::Data_row::get_int(const wxString &field){
    if(!this->check_row_idx_())
        return 0;
    int idx = this->col_idx(field);
    if(idx >= 0)
        return std::atoi(PQgetvalue(this->result_, this->current_, idx));

    return 0;
}


double
wxPg::Data_row::get_float(int idx){
    if(!this->check_row_idx_())
        return 0.0;
    if(!this->check_col_idx_(idx))
        return 0.0;

    return std::atof(PQgetvalue(this->result_, this->current_, idx));
}


double
wxPg::Data_row::get_float(const wxString &field){
    if(!this->check_row_idx_())
        return 0.0;
    int idx = this->col_idx(field);
    if(idx >= 0)
        return std::atof(PQgetvalue(this->result_, this->current_, idx));

    return 0.0;
}


const wxString
wxPg::Data_row::get_str(int idx){
    if(!this->check_row_idx_())
        return wxEmptyString;
    if(!this->check_col_idx_(idx))
        return wxEmptyString;

    return this->db_->char_to_str(PQgetvalue(this->result_,
                                  this->current_, idx));
}


const wxString
wxPg::Data_row::get_str(const wxString &field){
    if(!this->check_row_idx_())
        return wxEmptyString;
    int idx = this->col_idx(field);
    if(idx >= 0)
        return this->db_->char_to_str(PQgetvalue(this->result_,
                                                 this->current_, idx));

    return wxEmptyString;
}


const wxDateTime
wxPg::Data_row::get_date(int idx){
    wxDateTime result;
    if(!this->check_row_idx_())
        return result;
    if(!this->check_col_idx_(idx))
        return result;

    result.ParseDate(this->db_->char_to_str(PQgetvalue(this->result_,
                                                       this->current_, idx)));

    return result;
}


const wxDateTime
wxPg::Data_row::get_date(const wxString &field){
    wxDateTime result;
    if(!this->check_row_idx_())
        return result;
    int idx = this->col_idx(field);
    if(idx >= 0)
        result.ParseDate(this->db_->char_to_str(
                PQgetvalue(this->result_, this->current_, idx)));

    return result;
}


const wxDateTime
wxPg::Data_row::get_time(int idx){
    wxDateTime result;
    if(!this->check_row_idx_())
        return result;
    if(!this->check_col_idx_(idx))
        return result;

    result.ParseTime(this->db_->char_to_str(
            PQgetvalue(this->result_, this->current_, idx)));

    return result;
}


const wxDateTime
wxPg::Data_row::get_time(const wxString &field){
    wxDateTime result;
    if(!this->check_row_idx_())
        return result;
    int idx = this->col_idx(field);
    if(idx >= 0)
        result.ParseTime(this->db_->char_to_str(
                PQgetvalue(this->result_, this->current_, idx)));

    return result;
}


const wxDateTime
wxPg::Data_row::get_date_time(int idx){
    wxDateTime result;
    if(!this->check_row_idx_())
        return result;
    if(!this->check_col_idx_(idx))
        return result;

    result.ParseDateTime(this->db_->char_to_str(
            PQgetvalue(this->result_, this->current_, idx)));

    return result;
}


const wxDateTime
wxPg::Data_row::get_date_time(const wxString &field){
    wxDateTime result;
    if(!this->check_row_idx_())
        return result;
    int idx = this->col_idx(field);
    if(idx >= 0)
        result.ParseDateTime(this->db_->char_to_str(
                PQgetvalue(this->result_, this->current_, idx)));

    return result;
}


bool
wxPg::Data_row::get_bool(int idx){
    if(!this->check_row_idx_())
        return false;
    if(!this->check_col_idx_(idx))
        return false;
    return wxPg::cstr_to_bool(PQgetvalue(this->result_, this->current_, idx),
                    wxPg::oid_to_type_simple(PQftype(this->result_, idx)));
}


bool
wxPg::Data_row::get_bool(const wxString &field){
    if(!this->check_row_idx_())
        return false;
    int idx = this->col_idx(field);
    if(idx < 0)
        return false;

    return wxPg::cstr_to_bool(PQgetvalue(this->result_, this->current_, idx),
                    wxPg::oid_to_type_simple(PQftype(this->result_, idx)));
}


bool
wxPg::Data_row::check_col_idx_(int idx){
    int cols = this->cols();
    if(!cols)
        return false;

    if(idx >= 0 && idx < cols)
        return true;

    return false;
}


bool
wxPg::Data_row::check_row_idx_(){
    if(!this->result_)
        return false;
    if(this->current_ < 0)
        return false;
    int rows = PQntuples(this->result_);
    if(rows <= 0)
        return false;
    return this->current_ < rows;
}
