/* database.cpp - implementation of class wxPg::Database for connecting to
    database and executing SQL statements

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/db/database.h>
#include<wx/intl.h>

wxPg::Database::Database()
    : connection_(NULL), transaction_(false)
{}


wxPg::Database::Database(const wxString &coninfo)
    : connection_(NULL), coninfo_(coninfo), transaction_(false)
{
    this->connect();
}


wxPg::Database::~Database(){
    if(this->in_transaction())
        this->rollback();
    this->connect(false);
}


void
wxPg::Database::connection_info(const wxString &coninfo){
    this->coninfo_ = coninfo;
}


void
wxPg::Database::attach(wxPg::DB_client *cli, int level){
    wxPg::Database::client_ client = {cli, level};
    this->clients_.push_back(client);
}


void
wxPg::Database::detach(wxPg::DB_client *cli, int level){
    std::list<wxPg::Database::client_>::iterator i = this->clients_.begin();
    while(i != this->clients_.end())
        if(i->cli == cli && i->level == level){
            this->clients_.erase(i);
            return;
        }
        else ++i;
}


void
wxPg::Database::detach_all(int level){
    std::list<wxPg::Database::client_>::iterator i = this->clients_.begin();
    while(i != this->clients_.end())
        if(i->level == level)
            i = this->clients_.erase(i);
        else ++i;
}


const wxString&
wxPg::Database::connection_info() const{
    return this->coninfo_;
}


bool
wxPg::Database::connect(bool connect){
    if(connect){
        if(this->connected())
            return true;
        this->connection_ = PQconnectdb(this->coninfo_.char_str());
        if(!this->connection_)
            return false;
        if(PQstatus(this->connection_) == CONNECTION_OK)
            return true;
        PQfinish(this->connection_);
        this->connection_ = NULL;
        return false;
    }else if(this->connected()){
        this->rollback();
        PQfinish(this->connection_);
        this->connection_ = NULL;
        this->transaction_ = false;
        this->trans_depth_ = 0;
    }

    return true;
}


bool
wxPg::Database::connected() const{
    return this->connection_;
}


PGconn*
wxPg::Database::handle(){
    return this->connection_;
}


bool
wxPg::Database::begin_transaction(){
    if(this->transaction_)
        this->rollback();
    this->transaction_ = this->run(wxT("BEGIN"));
    if(this->transaction_)
        this->trans_depth_ = 1;
    return this->transaction_;
}


bool
wxPg::Database::enter_transaction(){
    if(this->transaction_){
        ++this->trans_depth_;
        return true;
    }

    return this->begin_transaction();
}


bool
wxPg::Database::commit(){
    if(this->transaction_){
        if(this->trans_depth_ == 1){
            if(this->run(wxT("COMMIT"))){
                this->transaction_ = false;
                this->trans_depth_ = 0;
                this->detach_clients_(wxPg::Database::ATTACH_TRANSACTION);
                return true;
            }
        }
        else --this->trans_depth_;
    }

    return false;
}


void
wxPg::Database::rollback(){
    if(this->transaction_){
        this->run(wxT("ROLLBACK"));
        this->transaction_ = false;
        this->trans_depth_ = 0;
        this->detach_clients_(wxPg::Database::ATTACH_TRANSACTION);
    }
}


bool
wxPg::Database::in_transaction(){
    return this->transaction_;
}


bool
wxPg::Database::run(const wxString &sql){
    PGresult *res = this->run_(sql);
    if(!res)
        return false;
    PQclear(res);
    return true;
}


bool
wxPg::Database::run(wxPg::Cached_result &result, const wxString &sql){
    PGresult *res = this->run_(sql);
    if(!res)
        return false;
    result.store(*this, res);
    PQclear(res);
    return true;
}


const wxString&
wxPg::Database::last_err() const{
    return this->err_;
}


void
wxPg::Database::sql_monitor(const wxString&){}


const wxString
wxPg::Database::char_to_str(const char *s)const{
    return wxString::From8BitData(s);
}


PGresult*
wxPg::Database::run_(const wxString &sql){
    this->sql_monitor(sql);
    if(!this->connected())
        return NULL;
    if(sql == wxEmptyString)
        return NULL;
    PGresult *result = PQexec(this->connection_, sql.char_str());
    if(!result){
        this->err_ = _("Fatal error\n");
        return NULL;
    }
    switch(PQresultStatus(result)){
    case PGRES_COMMAND_OK:
    case PGRES_TUPLES_OK:
        this->err_ = wxEmptyString;
        return result;
    //case PGRES_COPY_OUT:      to be implemented
    //case PGRES_COPY_IN:       to be implemented
    case PGRES_BAD_RESPONSE:
    case PGRES_NONFATAL_ERROR:
    case PGRES_FATAL_ERROR:
        this->err_ = wxString::From8BitData(PQresultErrorMessage(result));
        return NULL;
    default:
        this->err_ = _("Unknown error\n");
        return NULL;
    }
}


void
wxPg::Database::detach_clients_(int level){
    std::list<wxPg::Database::client_>::iterator i = this->clients_.begin();
    while(i != this->clients_.end())
        if(i->level == level){
            DB_client *cli = i->cli;
            i = this->clients_.erase(i);
            cli->detach();
        }else ++i;
}
