/* prepared_statement.cpp - prepared statement class wxPg::Prepared_statement

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/db/prepared_statement.h>
#include<wxPg/utils/utils.h>
#include<wxPg/utils/type.h>


wxPg::Prepared_statement::Prepared_statement()
: db_(NULL), prepared_(false)
{}


wxPg::Prepared_statement::Prepared_statement(wxPg::Database *db)
: db_(db), prepared_(false)
{}


wxPg::Prepared_statement::Prepared_statement(const wxString &name)
: db_(NULL), name_(name), prepared_(false)
{}


wxPg::Prepared_statement::Prepared_statement(wxPg::Database *db,
                                             const wxString &name)
: db_(db), name_(name), prepared_(false)
{}


wxPg::Prepared_statement::~Prepared_statement(){
    this->deallocate();
}


void
wxPg::Prepared_statement::db(wxPg::Database *db){
    if(this->prepared())
        return;

    this->db_ = db;
}


void
wxPg::Prepared_statement::name(const wxString &name){
    if(this->prepared())
        return;

    this->name_ = name;
}


bool
wxPg::Prepared_statement::prepare(){
    if(this->prepared())
        return false;
    if(!this->db_)
        return false;
    wxString sql = this->make_prepare_sql();
    if(sql == wxEmptyString){
        this->err_ = wxT("Empty SQL.");
    }

    this->prepared_ = this->db_->run(sql);
    if(!this->prepared_)
        this->err_ = this->db_->last_err();
    else this->db_->attach(this, wxPg::Database::ATTACH_CONNECTION);

    return this->prepared_;
}



bool
wxPg::Prepared_statement::execute(){
    if(!this->prepared()){
        this->err_ = wxT("Statement not prepared.");
        return false;
    }
    if(!this->db_){
        this->err_ = wxT("No database connection.");
        return false;
    }

    if(this->db_->run(this->make_execute_sql())){
        this->params_.clear();
        return true;
    }

    this->err_ = this->db_->last_err();
    return false;
}


void
wxPg::Prepared_statement::deallocate(){
    if(!this->prepared())
        return;
    if(!this->db_)
        return;

    this->db_->run(wxT("DEALLOCATE ") + this->name());
}


void
wxPg::Prepared_statement::detach(){
    this->prepared_ = false;
}


void
wxPg::Prepared_statement::statement(const wxString &sql){
    if(this->prepared())
        return;

    this->sql_ = sql;
}


const wxString
wxPg::Prepared_statement::make_prepare_sql(){
    if(this->statement() == wxEmptyString)
        return wxEmptyString;

    wxString res(wxT("PREPARE "));
    if(this->name() == wxEmptyString){
        this->generated_name_ = wxPg::Prepared_statement::generate_name_();
        this->name(this->generated_name_);
    }
    res += this->name();
    if(this->param_defs_.size() < this->params_.size())
        this->param_defs_.resize(this->params_.size(), wxPg::TYPE_UNKNOWN);
    if(this->param_defs_.size()){
        wxString types(this->type_name_(this->param_defs_[0]));
        for(unsigned i = 1; i < this->param_defs_.size(); ++i)
            types += wxT(", ") + this->type_name_(this->param_defs_[i]);
        res += wxT(" (") + types + wxT(") ");
    }

    res += wxT(" AS ") + this->statement();
    return res;
}


const wxString
wxPg::Prepared_statement::type_name_(int type){
    switch(type){
    case TYPE_INT:
        return wxT("int");
    case TYPE_FLOAT:
        return wxT("numeric");
    case TYPE_STRING:
        return wxT("text");
    case TYPE_DATE:
        return wxT("date");
    case TYPE_TIME:
        return wxT("time");
    case TYPE_TIMESTAMP:
        return wxT("timestamp");
    case TYPE_BOOL:
        return wxT("boolean");
    case TYPE_UNKNOWN:
    default:
        return wxT("unknown");
    }
}



int
wxPg::Prepared_statement::name_generator_;


const wxString
wxPg::Prepared_statement::generate_name_(){
    wxString res(wxT("wxPgStatement"));
    res << wxPg::Prepared_statement::name_generator_++;
    return res;
}


void
wxPg::Prepared_statement::param_def(int type, int idx){
    if(idx < 0)
        this->param_defs_.push_back(type);
    else{
        if(unsigned(idx) >= this->param_defs_.size())
            this->param_defs_.resize(idx + 1, wxPg::TYPE_UNKNOWN);
        this->param_defs_[idx] = type;
    }
}


int
wxPg::Prepared_statement::param_def_get(unsigned idx){
    if(idx < this->param_defs_.size())
        return this->param_defs_[idx];
    return wxPg::TYPE_UNKNOWN;
}


void
wxPg::Prepared_statement::param_def_truncate(unsigned size){
    if(size < this->param_defs_.size())
        this->param_defs_.resize(size);
}


void
wxPg::Prepared_statement::param_set(int value, int idx){
    if(!this->db_)
        return;
    wxString val;
    unsigned param_idx = (idx < 0) ? this->params_.size() : idx;
    switch(this->param_def_get(param_idx)){
    case wxPg::TYPE_BOOL:
        if(value)
            val = wxT("true");
        else val = wxT("false");
        break;
    case wxPg::TYPE_DATE:
    case wxPg::TYPE_TIME:
    case wxPg::TYPE_TIMESTAMP:
        val = wxT("NULL");
        break;
    default:
        val << value;
    }
    this->param_set_(val, idx);
}


void
wxPg::Prepared_statement::param_set(double value, int idx){
    if(!this->db_)
        return;
    wxString val;
    unsigned param_idx = (idx < 0) ? this->params_.size() : idx;
    switch(this->param_def_get(param_idx)){
    case wxPg::TYPE_BOOL:
        if(value)
            val = wxT("true");
        else val = wxT("false");
        break;
    case wxPg::TYPE_INT:
        val << int(value);
        break;
    case wxPg::TYPE_DATE:
    case wxPg::TYPE_TIME:
    case wxPg::TYPE_TIMESTAMP:
        val = wxT("NULL");
        break;
    default:
        val << value;
    }
    this->param_set_(val, idx);
}


void
wxPg::Prepared_statement::param_set(const wxString &value, int idx){
    if(!this->db_)
        return;
    wxString val;
    unsigned param_idx = (idx < 0) ? this->params_.size() : idx;
    switch(this->param_def_get(param_idx)){
    case wxPg::TYPE_BOOL:
        if(value.Len() > 4)
            val = wxT("NULL");
        else{
            wxString tmp(value.Lower());
            if(tmp == wxT("t") || tmp == wxT("true"))
                val = wxT("true");
            else if(tmp == wxT("f") || tmp == wxT("false"))
                val = wxT("false");
            else val = wxT("NULL");
        }
        break;
    case wxPg::TYPE_INT:
        {
            long int tmp;
            if(value.ToLong(&tmp))
                val << tmp;
            else val = wxT("NULL");
        }
        break;
    case wxPg::TYPE_FLOAT:
        {
            double tmp;
            if(value.ToDouble(&tmp))
                val << tmp;
            else val = wxT("NULL");
        }
        break;
    case wxPg::TYPE_DATE:
        {
            wxDateTime tmp;
            if(tmp.ParseDate(value.c_str()))
                val = tmp.Format(wxT("'%Y.%m.%d'"));
            else val = wxT("NULL");
        }
        break;
    case wxPg::TYPE_TIME:
        {
            wxDateTime tmp;
            if(tmp.ParseTime(value.c_str()))
                val = tmp.Format(wxT("'%H:%M:%S'"));
            else val = wxT("NULL");
        }
        break;
    case wxPg::TYPE_TIMESTAMP:
        {
            wxDateTime tmp;
            if(tmp.ParseDateTime(value.c_str()))
                val = tmp.Format(wxT("'%Y.%m.%d %H:%M:%S'"));
            else val = wxT("NULL");
        }
        break;
    default:
        val = wxT("'") + wxPg::escape_string(*this->db_, value) + wxT("'");
    }
    this->param_set_(val, idx);
}


void
wxPg::Prepared_statement::param_set(const wxDateTime &value, int idx){
    if(!this->db_)
        return;
    wxString val;
    unsigned param_idx = (idx < 0) ? this->params_.size() : idx;
    switch(this->param_def_get(param_idx)){
    case wxPg::TYPE_BOOL:
    case wxPg::TYPE_INT:
    case wxPg::TYPE_FLOAT:
        val = wxT("NULL");
        break;
    case wxPg::TYPE_DATE:
        val = value.Format(wxT("'%Y.%m.%d'"));
        break;
    case wxPg::TYPE_TIME:
        val = value.Format(wxT("'%H:%M:%S'"));
        break;
    default:
        val = value.Format(wxT("'%Y.%m.%d %H:%M:%S'"));
    }
    this->param_set_(val, idx);
}


void
wxPg::Prepared_statement::param_set(bool value, int idx){
    if(!this->db_)
        return;
    wxString val;
    unsigned param_idx = (idx < 0) ? this->params_.size() : idx;
    switch(this->param_def_get(param_idx)){
    case wxPg::TYPE_BOOL:
        if(value)
            val = wxT("true");
        else val = wxT("false");
        break;
    case wxPg::TYPE_INT:
    case wxPg::TYPE_FLOAT:
        if(value)
            val << 1;
        else val << 0;
        break;
    case wxPg::TYPE_STRING:
        if(value)
            val = wxT("'t'");
        else val = wxT("'f'");
        break;
    default:
        val = wxT("NULL");
    }
    this->param_set_(val, idx);
}


void
wxPg::Prepared_statement::param_set_null(int idx){
    if(!this->db_)
        return;
    this->param_set_(wxT("NULL"), idx);
}


void
wxPg::Prepared_statement::param_set_(const wxString &val, int idx){
    if(idx < 0)
        this->params_.push_back(val);
    else if(this->params_.size() >= unsigned(idx))
        this->params_[idx] = val;
    else{
        if(unsigned(idx) > this->params_.size())
            this->params_.resize(idx, wxT("NULL"));
        this->params_.push_back(val);
    }
}


const wxString
wxPg::Prepared_statement::make_execute_sql(){
    if(this->param_defs_.size() > this->params_.size())
        this->params_.resize(this->param_defs_.size());

    if(!this->params_.size())
        return wxT("EXECUTE ") + this->name();

    wxString params = this->params_[0];
    for(unsigned i = 1; i < this->params_.size(); ++i)
        params += wxT(", ") + this->params_[i];
    return wxT("EXECUTE ") + this->name() + wxT("(") + params + wxT(")");
}
