/* binding_macros.h - helper macros for binding wxWidgets controls

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __BINDING_MACROS_H
#define __BINDING_MACROS_H

#include "wxPg/binding/binding_ctrl.h"
#include "wxPg/binding/binding_list.h"
#include "wxPg/binding/grid_binding.h"

/** \file
    Binding macros to easily bind common controls.\n\n
    Example for using binding macros to bin controls to Data_row_provider
    (query is wxPg::Query, values of current row will be displayed in
    controls):
    \code
    wxPG_BIND_CONTROLS(&query)
        wxPG_BIND_TEXT_CTRL(wxTextCtrl1, wxT("FIRSTNAME"))
        wxPG_BIND_TEXT_CTRL(wxTextCtrl2, wxT("SECONDNAME"))
        wxPG_BIND_DATE_PICKER_CTRL(wxDateCtrl1, wxT("BIRTHDATE"))
        wxPG_BIND_SPIN_CTRL(wxSpinCtrl1, wxT("CATEGORY"))
    wxPG_BINDING_END()
    \endcode
    Example for using binding macros to bin list controls
    (query is wxPg::Query, values of columns will be contained by
    controls):
    \code
    wxPG_BIND_LIST_CONTROLS(&query)
        wxPG_BIND_LIST(wxChoice1, wxT("COUNTRY"));
        wxPG_BIND_LIST(wxListBox1, wxT("REGION"));
        wxPG_BIND_LIST(wxComboBox1, wxT("POST_CODE"));
    wxPG_BINDING_END()
    \endcode
    Remember to call update_UI() at the end.
*/

/** Start binding controls to binding. */
#ifndef wxPG_BIND_CONTROLS
    #define wxPG_BIND_CONTROLS(binding) { wxPg::Row_binding *b = binding;
#endif


/** Start binding controls with items to binding. */
#ifndef wxPG_BIND_LIST_CONTROLS
    #define wxPG_BIND_LIST_CONTROLS(binding) { wxPg::List_binding *b = binding;
#endif


/** End control binding. */
#ifndef wxPG_BINDING_END
    #define wxPG_BINDING_END() }
#endif

/** Bind wxCalendarCtrl to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_CALENDAR_CTRL
    #define wxPG_BIND_CALENDAR_CTRL(calendar, field) \
            b->bind((calendar), (field), wxPg::update_calendar_ctrl);
#endif


/** Bind wxComboBox to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_COMBO_BOX
    #define wxPG_BIND_COMBO_BOX(combo, field) \
            b->bind((combo), (field), wxPg::update_combo_box);
#endif


/** Bind wxComboCtrl to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_COMBO_CTRL
    #define wxPG_BIND_COMBO_CTRL(combo, field) \
            b->bind((combo), (field), wxPg::update_combo_ctrl);
#endif


/** Bind wxRichTextCtrl to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_RICH_TEXT_CTRL
    #define wxPG_BIND_RICH_TEXT_CTRL(text, field) \
            b->bind((text), (field), wxPg::update_rich_text_ctrl);
#endif


/** Bind wxTextCtrl to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_TEXT_CTRL
    #define wxPG_BIND_TEXT_CTRL(text, field) \
            b->bind((text), (field), wxPg::update_text_ctrl);
#endif


/** Bind wxSpinCtrl to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_SPIN_CTRL
    #define wxPG_BIND_SPIN_CTRL(spin, field) \
            b->bind((spin), (field), wxPg::update_spin_ctrl);
#endif


/** Bind wxStaticText to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_STATIC_TEXT
    #define wxPG_BIND_STATIC_TEXT(static_text, field) \
            b->bind((static_text), (field), wxPg::update_static_text);
#endif


/** Bind wxHyperlinkCtrl to wxPg::Data_row_provider.
    This updates URL of wxHyperlinkCtrl, label is left unaffected.
*/
#ifndef wxPG_BIND_HYPERLINK_CTRL
    #define wxPG_BIND_HYPERLINK_CTRL(hyperlink, field) \
            b->bind((hyperlink), (field), wxPg::update_hyperlink_ctrl);
#endif


/** Bind wxDatePickerCtrl to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_DATE_PICKER_CTRL
    #define wxPG_BIND_DATE_PICKER_CTRL(date_ctrl, field) \
            b->bind((date_ctrl), (field), wxPg::update_date_picker_ctrl);
#endif

/** Bind wxCheckBox to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_CHECK_BOX
    #define wxPG_BIND_CHECK_BOX(check_box, field) \
            b->bind((check_box), (field), wxPg::update_check_box);
#endif


/** Bind wxGauge to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_GAUGE
    #define wxPG_BIND_GAUGE(gauge, field) \
            b->bind((gauge), (field), wxPg::update_gauge);
#endif


/** Bind wxSpinButton to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_SPIN_BUTTON
    #define wxPG_BIND_SPIN_BUTTON(spin, field) \
            b->bind((spin), (field), wxPg::update_spin_button);
#endif


/** Bind wxItemContainer to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_LIST_ITEM
    #define wxPG_BIND_LIST_ITEM(control, field) \
            b->bind((control), (field), wxPg::update_item_container);
#endif


/** Bind wxRadioBox to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_RADIO_BOX
    #define wxPG_BIND_RADIO_BOX(control, field) \
            b->bind((control), (field), wxPg::update_radio_box);
#endif


/** Bind wxVListBox to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_VLIST_BOX
    #define wxPG_BIND_VLIST_BOX(control, field) \
            b->bind((control), (field), wxPg::update_vlist_box);
#endif


/** Bind wxHtmlWindow to wxPg::Data_row_provider. */
#ifndef wxPG_BIND_HTML_WINDOW
    #define wxPG_BIND_HTML_WINDOW(control, field) \
            b->bind((control), (field), wxPg::update_html_window);
#endif


/** Bind wxItemContainer to wxPg::Result_data_provider. */
#ifndef wxPG_BIND_LIST
    #define wxPG_BIND_LIST(control, field) \
            wxPg::bind_list_str(b, control, field);
#endif

#endif
