/* data_row_provider.h - class wxPg::Data_row_provider, interface for
    accessing data from single row of SQL query result.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DATA_ROW_PROVIDER
#define __DATA_ROW_PROVIDER

#include "wxPg/conf.h"
#include<wx/string.h>
#include<wx/datetime.h>

namespace wxPg{

/** Interface, representing a single row of SQL query result. */
class wxPG_DECLSPEC
Data_row_provider{
public:
    /** Destructor. */
    virtual
    ~Data_row_provider(){}

    /** Get number of columns in a row.
        @return number of columns in a row.
    */
    virtual int
    cols() = 0;

    /** Get field name by column index.
        @param idx - column index.
        @return field name for column idx.
    */
    virtual const wxString
    col_name(int idx) = 0;

    /** Get column index by name.
        @param field - field name.
        @return 0 - based column index, negative value for non existing field.
    */
    virtual int
    col_idx(const wxString &field) = 0;

    /** Returns type of column (for built in types only).
        If type of column is not built in or there is no column
        whith such index, wxPg::TYPE_UNKNOWN is returned. See type.h.
        @param idx - column index.
        @return one of types, defined in type.h.
    */
    virtual int
    type_simple(int idx) = 0;

    /** Returns type of field (for built in types only).
        If type of column is not built in or there is no such field ir a row,
        wxPg::TYPE_UNKNOWN is returned. See type.h.
        @param field - field name.
        @return one of types, defined in type.h.
    */
    virtual int
    type_simple(const wxString &field) = 0;

    /** Check if field value is NULL.
        @param idx - column index.
        @return true if value is NULL or field not exists, false otherwise.
    */
    virtual bool
    null(int idx) = 0;

    /** Check if field value is NULL.
        @param field - field name.
        @return true if value is NULL or field not exists, false otherwise.
    */
    virtual bool
    null(const wxString &field) = 0;

    /** Returns column value as integer.
        If value is NULL and in case of error 0 is returned.
        @param idx - column index.
        @return integer representation of column value.
    */
    virtual int
    get_int(int idx) = 0;

    /** Returns column value as integer.
        If value is NULL and in case of error 0 is returned.
        @param field - field name.
        @return integer representation of column value.
    */
    virtual int
    get_int(const wxString &field) = 0;

    /** Returns column value as double.
        If value is NULL and in case of error 0.0 is returned.
        @param idx - column index.
        @return double representation of column value.
    */
    virtual double
    get_float(int idx) = 0;

    /** Returns column value as double.
        If value is NULL and in case of error 0.0 is returned.
        @param field - field name.
        @return double representation of column value.
    */
    virtual double
    get_float(const wxString &field) = 0;

    /** Returns column value as wxString.
        If value is NULL and in case of error empty string is returned.
        @param idx - column index.
        @return string representation of column value.
    */
    virtual const wxString
    get_str(int idx) = 0;

    /** Returns column value as wxString.
        If value is NULL and in case of error empty string is returned.
        @param field - field name.
        @return string representation of column value.
    */
    virtual const wxString
    get_str(const wxString &field) = 0;

    /** Returns column value as date.
        If value is NULL and in case of error new wxDateTime
        object is returned.
        @param idx - column index.
        @return date representation of column value.
    */
    virtual const wxDateTime
    get_date(int idx) = 0;

    /** Returns column value as date.
        If value is NULL and in case of error new wxDateTime
        object is returned.
        @param field - field name.
        @return date representation of column value.
    */
    virtual const wxDateTime
    get_date(const wxString &field) = 0;

    /** Returns column value as time.
        If value is NULL and in case of error new wxDateTime
        object is returned.
        @param idx - column index.
        @return time representation of column value.
    */
    virtual const wxDateTime
    get_time(int idx) = 0;

    /** Returns column value as time.
        If value is NULL and in case of error new wxDateTime
        object is returned.
        @param field - field name.
        @return time representation of column value.
    */
    virtual const wxDateTime
    get_time(const wxString &field) = 0;

    /** Returns column value as date&time.
        If value is NULL and in case of error new wxDateTime
        object is returned.
        @param idx - column index.
        @return date&time representation of column value.
    */
    virtual const wxDateTime
    get_date_time(int idx) = 0;

    /** Returns column value as date&time.
        If value is NULL and in case of error new wxDateTime
        object is returned.
        @param field - field name.
        @return date&time representation of column value.
    */
    virtual const wxDateTime
    get_date_time(const wxString &field) = 0;

    /** Returns column value as bool (intended for boolean type columns).
        NULL is interpreted as false.\n
        For numeric fields, 0 is interpreted as false, anything else as true.\n
        For string fields, empty string is false, non empty is true.
        @param idx - column index.
        @return bool representation of column value.
    */
    virtual bool
    get_bool(int idx) = 0;

    /** Returns column value as bool (intended for boolean type columns).
        NULL is interpreted as false.\n
        For numeric fields, 0 is interpreted as false, anything else as true.\n
        For string fields, empty string is false, non empty is true.
        @param field - field name.
        @return bool representation of column value.
    */
    virtual bool
    get_bool(const wxString &field) = 0;
};

}

#endif
