/* database.h - class wxPg::Database for connecting to database and
    executing SQL statements

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DATABASE_H
#define __DATABASE_H

#include "wxPg/conf.h"
#include "wxPg/db/db_client.h"
#include "wxPg/db/cached_result.h"
#include<wx/string.h>
#include<list>


namespace wxPg{

//forward declaration
class Query;

/** Class for connecting to database and executing SQL statements. */
class wxPG_DECLSPEC
Database{
public:
    friend class wxPg::Query;

    /** Create new database object.
        Call connection_info() to complete initialization.
    */
    Database();

    /** Creates new object and initializes it with specified connection info.
        @param coninfo - see PQconnectdb() in PostgreSQL documentation for description.
    */
    explicit
    Database(const wxString &coninfo);

    /** Destroys object. */
    virtual
    ~Database();

    /** Sets connection info for this object.
        @param coninfo - see PQconnectdb() in PostgreSQL documentation for description.
    */
    void
    connection_info(const wxString &coninfo);

    /** Return current connection info.
        See PQconnectdb() in PostgreSQL documentation for returned string
        description.
    */
    const wxString&
    connection_info() const;

    /** Connect to or disconnect from database.
        @param connect - if is true, than connect, else - disconnect.
        @return true on success and false on error.
    */
    bool
    connect(bool connect = true);

    /** Is this object connected to database.
        @return true, if connected, else otherwise.
    */
    bool connected()const;

    /** Return connection handle. */
    PGconn*
    handle();

    /** Attach DB_client to connection.
        Client is detached when database is disconnected.
    */
    static const int
    ATTACH_CONNECTION = 10;

    /** Attach DB_client to transaction.
        Client is detached when transaction is committed or rollbacked.
    */
    static const int
    ATTACH_TRANSACTION = 20;

    /** Attach new client to this connection.
        When database is disconnected or transaction is ended,
        detach() method is called for each client.
        @param cli - a client to attach.
        @param level - level of attach, use one of ATTACH_* constants.
    */
    void
    attach(DB_client *cli, int level);

    /** Detach client.
        Note, that this method does not call detach() on a client.
        @param cli - a client to detach.
        @param level - level of detach, same as attach level for attach().
    */
    void
    detach(DB_client *cli, int level);

    /** Detach all clients.
        Note, that this method does not call detach() on the clients.
        @param level - level of detach, same as attach level for attach().
    */
    void
    detach_all(int level);

    /** Begin new transaction.
        If a transaction is already started, rollbacks old and starts new.
        Call last_err() to get error message.
        @return true on success, and false on error.
    */
    bool
    begin_transaction();

    /** Ensure, that database is in transaction.
        If database is not in transaction, starts new transaction,
        simply returns true otherwise.
        @return false on error, true if database is now in transaction.
    */
    bool
    enter_transaction();

    /** Commit current transaction.
        If return value is false, call last_err() to get error message.
        @return true on success and false on error or if not in transaction.
    */
    bool
    commit();

    /** Rollbacks current transaction.
        Does nothing if not in transaction.
    */
    void
    rollback();

    /** Is database in transaction.
        @return true if transaction is started and false otherwise.
    */
    bool
    in_transaction();

    /** Run SQL statement.
        If returns false, call last_err() to get error message.
        Don't use it for SQL statements, that return data,
        as there is no way to get it.
        @param sql - SQL statement to execute.
        @return true on success and false on failure.
    */
    bool
    run(const wxString &sql);

    /** Run SQL query.
        If returns false, call last_err() to get error message.
        @param result - Cached_result object, where result is stored.
        @param sql - SQL statement to execute.
        @return true on success and false on failure.
    */
    bool
    run(Cached_result &result, const wxString &sql);

    /** Get last error.
        @return message for last error.
    */
    const wxString&
    last_err() const;

    /** Monitor SQL for this database object.
        This method does nothing. Override it in derived class to get
        ability to monitor any SQL statement, executed by this object.
        @param sql - SQL statement, executed by this object.
    */
    virtual void
    sql_monitor(const wxString &sql);


    /** Convert C-string to wxString.
        Used internally for converting data, received from database,
        to wxString.
        @param s - zero terminated char array.
        @return s, converted to wxString.
    */
    const wxString
    char_to_str(const char *s)const;
private:
    //Database connection
    PGconn*
    connection_;

    wxString
    coninfo_,  //connection info
    err_;      //last error

    struct
    client_{
        DB_client *cli;
        int level;
    };

    std::list<client_>
    clients_;  //attached clients

    bool
    transaction_;  //is transaction started

    unsigned
    trans_depth_;  //number of enter_transaction()

    //internal SQL statement execution
    PGresult*
    run_(const wxString &sql);

    //no copy-constructor and assignement operator
    Database(const Database&){}
    Database &operator=(const Database&){return *this;}

    //detach all attached clients
    void
    detach_clients_(int level);
};

}

#endif
