/* prepared_statement.h - prepared statement class wxPg::Prepared_statement

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __PREPARED_STATEMENT_H
#define __PREPARED_STATEMENT_H

#include "wxPg/conf.h"
#include "wxPg/db/database.h"

namespace wxPg{

/** Prepared statement class.
    Prepared statement has its name. If no name is specified,
    it is autogenerated.
*/
class wxPG_DECLSPEC
Prepared_statement : public DB_client{
public:
    /** Creates new object.
        Call database() to complete initialization.
    */
    Prepared_statement();

    /** Creates new object with given database connection.
        @param db - database connection to be used by this object.
    */
    explicit
    Prepared_statement(Database *db);

    /** Creates new object with given name.
        Call database() to complete initialization.
        @param name - name for prepared statement.
    */
    explicit
    Prepared_statement(const wxString &name);

    /** Creates new object with give database connection and name.
        @param db - database connection to be used by this object.
        @param name - name for prepared statement.
    */
    Prepared_statement(Database *db, const wxString &name);

    /** Destructor. */
    virtual
    ~Prepared_statement();

    /** Set database connection for this object.
        You can not change connection while statement is prepared.
        @param db - database connection.
    */
    void
    db(Database *db);

    /** Return current database connection.
        @return current database connection.
    */
    Database*
    db()const{return this->db_;}

    /** Set name for prepared statement.
        This is optional. If no name is given, the name is automatically
        generated. You can not change name while statement is prepared.
        @param name - statment name.
    */
    void
    name(const wxString &name);

    /** Return statement name.
        If no name was given, stement has no name util it is prepared.
        @return statement name.
    */
    const wxString&
    name()const{return this->name_;}

    /** Prepare statement.
        This create a server side object, which is destroyed when database
        is disconnected or by calling deallocate(). If this method returns
        false, call last_err() to get error message.
        @return true on success, false on error.
    */
    virtual bool
    prepare();

    /** Construct EXECUTE statement. Used internally by this object.
        Use this method for statements that return result: this will
        return an sql statement that can be openned by wxPg::Query.
        @return Execute statement of empty string on error.
    */
    virtual const wxString
    make_execute_sql();

    /** Is statement prepared.
        @return true if statement is prepared, false otherwise.
    */
    bool
    prepared()const{return this->prepared_;}

    /** Execute statement.
        @return true on success, false on error.
    */
    virtual bool
    execute();

    /** Destroy the server side object.
        After this object can not be executed but can be reinitialized
        for a new statement.
    */
    virtual void
    deallocate();

    /** Return message for last error.
        @return last error message.
    */
    const wxString&
    last_err()const{return this->err_;}

    //inherited
    virtual void
    detach();

    /** SQL for statement.
        Specify only the statement itself, the full
        PREPARE statement will be constructed by this object.
        You can not change statement while it is prepared.
        @param sql - statement to be prepared.
    */
    void
    statement(const wxString &sql);

    /** Get current SQL statement.
        @return current SQL statement.
    */
    const wxString&
    statement()const{return this->sql_;}

    /** Define parametter type.
        Type definitions are automatically used when preparing statement.\n
        It is optional to define parametter types. Don't change parametter
        type after setting its value or set its value again after changing
        type.
        @param type - one of data types, defined in type.h.
        @param idx - parametter index, appended to the end, if negative.
    */
    virtual void
    param_def(int type, int idx = -1);

    /** Get data type of earlier defined parametter.
        For undefined parametters TYPE_UNKNOWN is returned.
        @param idx - parametter idx.
        @return one of types, defined in type.h.
    */
    virtual int
    param_def_get(unsigned idx);

    /** Truncate parametter definitions.
        Nothing is done, if truncating to bigger size than currently is.
        @param size - numer of arguments to leave.
    */
    virtual void
    param_def_truncate(unsigned size);

    /** Set parametter for statement.
        If idx is higher than current count of parametters,
        a necessary amount of NULL parametters is appended.
        @param value - value for parametter.
        @param idx - parametter index, append to the end if negative.
    */
    virtual void
    param_set(int value, int idx = -1);

    /** Set parametter for statement.
        If idx is higher than current count of parametters,
        a necessary amount of NULL parametters is appended.
        @param value - value for parametter.
        @param idx - parametter index, append to the end if negative.
    */
    virtual void
    param_set(double value, int idx = -1);

    /** Set parametter for statement.
        If idx is higher than current count of parametters,
        a necessary amount of NULL parametters is appended.
        @param value - value for parametter.
        @param idx - parametter index, append to the end if negative.
    */
    virtual void
    param_set(const wxString &value, int idx = -1);

    /** Set parametter for statement.
        If idx is higher than current count of parametters,
        a necessary amount of NULL parametters is appended.
        @param value - value for parametter.
        @param idx - parametter index, append to the end if negative.
    */
    virtual void
    param_set(const wxDateTime &value, int idx = -1);

    /** Set parametter for statement.
        If idx is higher than current count of parametters,
        a necessary amount of NULL parametters is appended.
        @param value - value for parametter.
        @param idx - parametter index, append to the end if negative.
    */
    virtual void
    param_set(bool value, int idx = -1);

    /** Set parametter for statement to NULL.
        If idx is higher than current count of parametters,
        a necessary amount of NULL parametters is appended.
        @param idx - parametter index, append to the end if negative.
    */
    virtual void
    param_set_null(int idx = -1);
protected:
    /** Construct PREPARE statement.
        @return PREPARE statement or empty string on error.
    */
    virtual const wxString
    make_prepare_sql();

    //autogenerated statement name
    wxString
    generated_name_;
private:
    Database*
    db_;     //database connection

    wxString
    name_,   //name of statement
    err_,    //last error message
    sql_;    //SQL statement

    bool
    prepared_;  //is statement prepared

    std::vector<wxString>
    params_;  //statement parametters

    std::vector<int>
    param_defs_;

    //no copy constructor
    Prepared_statement(const Prepared_statement&){}
    //no assignement operator
    Prepared_statement &operator=(const Prepared_statement&){return *this;}

    //name generator
    static int
    name_generator_;

    //generate statement name
    static const wxString
    generate_name_();

    //set value of parametter
    inline void
    param_set_(const wxString &val, int idx);

    //get type name from type id
    inline const wxString
    type_name_(int type);
};

} //namespace wxPg

#endif
