/* query.h - class wxPg::Query for executing SQL queries

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __QUERY_H
#define __QUERY_H

#include "wxPg/conf.h"
#include<climits>
#include "wxPg/db/database.h"
#include "wxPg/db/result_data.h"


namespace wxPg{

/** Class for executing SQL queries, returning result. */
class wxPG_DECLSPEC
Query : public Result_data{
public:
    /** Fetch and move actions. */
    enum{
        /** Fetch or move to first. */
        FM_FIRST = 1,
        /** Fetch or move to last. */
        FM_LAST,
        /** Fetch or move to row by absolute index. */
        FM_ABSOLUTE,
        /** Fetch or move to row by relative to current index. */
        FM_RELATIVE,
        /** Fetch all. */
        FM_ALL,
        /** Fetch or move in forward direction. */
        FM_FORWARD,
        /** Fetch all records in forward direction. */
        FM_FORWARD_ALL,
        /** Fetch or move in backward direction. */
        FM_BACKWARD,
        /** Fetch all in backward direction. */
        FM_BACKWARD_ALL
    };

    /** Name of portal, used by this query. Used internally. */
    const wxString
    PORTAL_NAME;

    /** Create new query object for database.
        @param db - database connection.
    */
    explicit
    Query(Database *db);

    /** Destroy query object, free resources. */
    virtual
    ~Query();

    /** Closes query and frees resources. */
    virtual void
    clear();

    /** Open query - execute sql statement.
        Call last_err() for error message.
        Call fetch() to get all or part of returned result.
        @param sql specifies an SQL statement.
        @return - true on success and false on error.
    */
    virtual bool
    open(const wxString &sql);

    /** Returns message for last error. */
    const wxString
    last_err() const;

    /** Fetch data of some rows.
        You need to call this method to obtain any data from query.
        @param rows - number of rows to be fetched. Ignored for some flags.
        @param flag sould be one of fetch and move actions.
        @return - true on success, false on error.
    */
    virtual bool
    fetch(int rows, int flag = FM_FORWARD);

    /** Move cursor to different row without fetching data.
        @param rows specifiels number of rows to move. Ignored for some flags.
        @param flag sould be one of fetch and move actions. Not all are
               accepted.
        @return - true on success, false on error.
    */
    virtual bool
    move(int rows, int flag = FM_FORWARD);
private:
    bool
    active_;   //is query openned

    //no copy-constructor or assignement
    Query(const Query &q) :Data_row(NULL), Result_data(NULL){}
    Query &operator=(const Query&){return *this;}

    //format fetch or move operaction and row count string
    const wxString
    fetch_move_(const wxString &prefix, int rows, int flag);

    //global variable for generating unique query portal names
    static unsigned
    portal_id_gen;

    //generate unique portal name
    static const wxString
    gen_portal_name();
};

}

#endif
