/* query_browser.h - class wxPg::Query_browser for executing and getting
   result from SQL queries.

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __QUERY_BROWSER
#define __QUERY_BROWSER

#include "wxPg/db/query.h"


namespace wxPg{

/** Class for executing SQL queries, that return large results.
    An object of this class keep a certain amount rows fetched to
    local buffer and alows working with entire result without worying
    about record fetching.
    NOTE: its not recomended to use fetch() and move() methods of this class.
*/
class wxPG_DECLSPEC
Query_browser : public Query{
public:
    /** Default buffer size.
        Such amount of rows is fetched by default from query.
    */
    static const unsigned DEF_BUFFER_SIZE = 50;

    /** Create new object.
        @param db - database connection.
        @param buf_size - buffer size, specifies a number of rows to be
            fetched to local buffer.
    */
    explicit
    Query_browser(Database *db, unsigned buf_size = DEF_BUFFER_SIZE);

    /** Destroys object. */
    virtual
    ~Query_browser();

    /** Change buffer size.
        It is strongly recomended NOT to call this method when Query_browser
        is open.
        @param size - new buffer size.
    */
    void
    buffer_size(unsigned size);

    /** Get buffer size.
        @return currently used size of buffer.
    */
    unsigned
    buffer_size()const;

    /** Go to first row in result. */
    virtual void
    first();

    /** Go to last row in result. */
    virtual void
    last();

    /** Go to next row in result.
        @return true when success, false on error or when end of result
            is reached (current row is already last).
    */
    virtual bool
    next();

    /** Go to previous row in result.
        @return true on success, false on error or when beginning of result
            is reached (current row is already the first one).
    */
    virtual bool
    prev();

    //inherited
    virtual bool
    open(const wxString &sql);

    //inherited
    virtual bool
    fetch(int rows, int flag = FM_FORWARD);

    //inherited
    virtual bool
    move(int rows, int flag = FM_FORWARD);

    //inherited
    using Query::current;
    virtual bool
    current(int new_curr);

    /** Absolute position in result.
        Don't mix this with current(), which returns position in current
            buffer.
        @return index of currently active row.
    */
    unsigned long long
    abs_pos()const;

    /** Database cursor position.
        This function returns position of cursor, used for reading data
        from result.
        @return current database cursor position.
    */
    long long
    cursor_pos()const;

    /** Is currently active row last in result.
        This method returns false until the end of result is reached and
        the number of rows in result is known.
        @return true if current row is last in result, false otherwise.
    */
    bool
    is_last()const;

    /** Get number of rows in result.
        The exact number of rows in result is unknown until the end of result
        is reached.
        @return currently known (exact if end was reached) number of rows in
            result.
    */
    unsigned long long
    record_count()const;
private:
    unsigned
    buffer_size_; //size of buffer

    unsigned long long
    abs_pos_,        //absolute position in result
    total_recs_;     //currently known number of rows in result

    long long
    abs_cursor_pos_; //absolute cursor position

    bool
    end_reached_;    //was end of result reached.

    inline bool
    fetch_first_();  //sub-function of fetch().

    inline bool
    fetch_last_();   //sub-function of fetch().

    inline bool
    fetch_absolute_(int row);  //sub-function of fetch().

    inline bool
    fetch_relative_(int offset);  //sub-function of fetch().

    inline bool
    fetch_forward_(int rows);   //sub-function of fetch().

    inline bool
    fetch_forward_all_();   //sub-function of fetch().

    inline bool
    fetch_backward_(int rows);  //sub-function of fetch().

    inline bool
    fetch_backward_all_();   //sub-function of fetch().
};

}

#endif
